@@ -1,6 +1,6 @@
 # -------------------------------------------------------------------------------
-# Licence:
-# Copyright (c) 2012-2019 Luzzi Valerio 
+# MIT License:
+# Copyright (c) 2012-2019 Luzzi Valerio
 #
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
@@ -30,8 +30,10 @@ import tempfile
 import hashlib
 import shutil
 import random
+from .module_types import parseInt, isstring, isarray
 from .module_log import Logger
 
+
 def now():
     """
     now
@@ -56,7 +58,8 @@ def normpath(pathname):
     if not pathname:
         return ""
     pathname = os.path.normpath(pathname.replace("\\", "/")).replace("\\", "/")
-    pathname = pathname.replace(":/", "://") #patch for s3:// and http:// https://
+    # patch for s3:// and http:// https://
+    pathname = pathname.replace(":/", "://")
     return pathname
 
 
@@ -87,6 +90,80 @@ def justfname(pathname):
     return normpath(os.path.basename(normpath(pathname)))
 
 
+# def israster(pathname):
+#     """
+#     israster
+#     """
+#     return pathname and os.path.isfile(pathname) and justext(pathname).lower() in ("tif",)
+
+# moved to module_s3.py
+# def isshape(pathname):
+#     """
+#     isshape
+#     """
+#     return pathname and os.path.isfile(pathname) and justext(pathname).lower() in ("shp",)
+
+def normshape(pathname):
+    """
+    normshape
+    """
+    if pathname is None:
+        return None
+    # sometime the shapefile is in the form s3://bucket/filename.shp|layername
+    if ".shp" in pathname.lower():
+        pathname = pathname.split("|", 1)[0]
+    return normpath(pathname)
+
+
+def parse_key_value(text, sep="="):
+    """
+    parse_key_value
+    """
+    if text is None:
+        return {}
+    elif isinstance(text, str):
+        key, value = text.split(sep, 1)
+        key, value = key.strip(), value.strip()
+        if "," in value:
+            value = listify(value)
+        return key, value
+    elif isinstance(text, (tuple, list)) and len(text) == 2:
+        return text
+    return None, None
+
+
+def parse_shape_path(pathname):
+    """
+    normshape
+    """
+    if pathname is None:
+        return None
+    # sometime the shapefile is in the form 
+    # s3://bucket/filename.shp|layername=filename
+    # s3://bucket/filename.shp|layername=filename|fid=0
+    # s3://bucket/filename.shp|layername=filename|fid=1,3,7
+    parts = pathname.split("|")
+    if len(parts) == 1:
+        return parts[0], None, None
+    elif len(parts) == 2:
+        filename = parts[0]
+        key, name = parse_key_value(parts[1])
+        layername = name if key.lower() == "layername" else None
+        fid = None
+        return filename, layername, fid
+    elif len(parts) == 3:
+        filename = parts[0]
+        key, name = parse_key_value(parts[1])
+        layername = name if key.lower() == "layername" else None
+        key, fid = parse_key_value(parts[2])
+        if isstring(fid) and key.lower() == "fid":
+            fid = parseInt(fid)   
+        elif isarray(fid) and key.lower() == "fid":
+            fid = [parseInt(f) for f in fid]
+        return filename, layername, fid
+    return None, None, None
+
+
 def justext(pathname):
     """
     justext
@@ -105,20 +182,6 @@ def forceext(pathname, newext):
     return normpath(pathname)
 
 
-def israster(pathname):
-    """
-    israster
-    """
-    return pathname and os.path.isfile(pathname) and justext(pathname).lower() in ("tif",)
-
-
-def isshape(pathname):
-    """
-    isshape
-    """
-    return pathname and os.path.isfile(pathname) and justext(pathname).lower() in ("shp",)
-
-
 def filesize(filename):
     """
     filesize
@@ -152,18 +215,19 @@ def mkdirs(pathname):
         Logger.error(ex)
     return os.path.isdir(pathname)
 
+
 def remove(pathname):
     """
     remove - remove a file
     """
     if pathname is None:
         return
-    elif os.path.isfile(pathname):
+    elif isinstance(pathname, str) and os.path.isfile(pathname):
         try:
             os.remove(pathname)
         except OSError as ex:
             Logger.error(ex)
-    elif os.path.isdir(pathname):
+    elif isinstance(pathname, str) and os.path.isdir(pathname):
         try:
             shutil.rmtree(pathname)
         except OSError as ex:
@@ -201,7 +265,7 @@ def strtofile(text, filename, append=False):
             text = text.encode("utf-8")
         if isinstance(text, (bytes,)):
             flag += 'b'
-        mkdirs(justpath(filename))
+        os.makedirs(justpath(filename), exist_ok=True)
         with open(filename, flag) as stream:
             if text:
                 stream.write(text)
@@ -241,7 +305,7 @@ def filetojson(filename):
         Logger.error(ex)
         return None
 
-    
+
 def listify(text, sep=",", trim=False):
     """
     listify -  make a list from string
@@ -285,7 +349,60 @@ def md5sum(filename):
                 break
             d.update(buf)
         f.close()
-        res=  d.hexdigest()
+        res = d.hexdigest()
         return res
     else:
         return ""
+
+
+def lock(filename, username):
+    """
+    lock - create a lock file
+    """
+    filelock = forceext(filename, "lock")
+    os.makedirs(justpath(filelock), exist_ok=True)
+    #get pid if username is not provided
+    username = username or f"{os.getpid()}"
+    with open(filelock, "w", encoding="utf-8") as f:
+        f.write(f"{username},{datetime.datetime.now()}")
+
+
+def unlock(filename):
+    """
+    unlock - remove the lock file
+    """
+    filelock = forceext(filename, "lock")
+    if os.path.isfile(filelock):
+        os.unlink(filelock)
+
+
+def is_locked(filename, username, timeout=60):
+    """
+    is_locked - check if the file is locked
+    """
+    locked = False
+    timeout = timeout or 60
+    filelock = forceext(filename, "lock")
+    if os.path.isfile(filelock):
+        with open(filelock, "r", encoding="utf-8") as f:
+            locker, locktime = f.read().split(",")
+            if locker != username:
+                locktime = datetime.datetime.strptime(locktime, "%Y-%m-%d %H:%M:%S.%f")
+                if (datetime.datetime.now() - locktime).total_seconds() < timeout:
+                    locked = True
+                else:
+                    # remove the lock file
+                    os.unlink(filelock)
+    return locked
+
+
+def locked_by(filename):
+    """
+    locked_by - get the locker
+    """
+    filelock = forceext(filename, "lock")
+    if os.path.isfile(filelock):
+        with open(filelock, "r", encoding="utf-8") as f:
+            locker, _ = f.read().split(",")
+            return locker
+    return None
