@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------
-# Licence:
+# MIT License:
 # Copyright (c) 2012-2022 Valerio for Gecosistema S.r.l.
 #
 # The above copyright notice and this permission notice shall be
@@ -70,15 +70,19 @@ def CalculateStats(ds):
     :return:
     """
     # Set the statistics
-    dtype = ds.GetRasterBand(1).DataType
-    arr = ds.GetRasterBand(1).ReadAsArray()
-    if dtype in (gdal.GDT_Float32, gdal.GDT_Float64):
-        arr[arr <= -9999] = np.nan
-        minValue = float(np.nanmin(arr))
-        maxValue = float(np.nanmax(arr))
-        meanValue = float(np.nanmean(arr))
-        stdValue = float(np.nanstd(arr))
-        ds.GetRasterBand(1).SetStatistics(minValue, maxValue, meanValue, stdValue)
+    if ds:
+        dtype = ds.GetRasterBand(1).DataType
+        arr = ds.GetRasterBand(1).ReadAsArray()
+        if dtype in (gdal.GDT_Float32, gdal.GDT_Float64):
+            arr[arr <= -9999] = np.nan
+            if np.any(~np.isnan(arr)):
+                minValue = float(np.nanmin(arr))
+                maxValue = float(np.nanmax(arr))
+                meanValue = float(np.nanmean(arr))
+                stdValue = float(np.nanstd(arr))
+            else:
+                minValue = maxValue = meanValue = stdValue = 0
+            ds.GetRasterBand(1).SetStatistics(minValue, maxValue, meanValue, stdValue)
 
 
 def GTiff2Cog(filetif, fileout=None, algo="NEAREST"):
@@ -193,10 +197,13 @@ def Numpy2GTiff(arr, gt, prj, fileout, format="GTiff", save_nodata_as=-9999, met
             if dtype in (gdal.GDT_Float32, gdal.GDT_Float64):
                 data = np.array(arr)
                 data[data == save_nodata_as] = np.nan
-                minValue = float(np.nanmin(data))
-                maxValue = float(np.nanmax(data))
-                meanValue = float(np.nanmean(data))
-                stdValue = float(np.nanstd(data))
+                if np.all(np.isnan(data)):
+                    minValue = maxValue = meanValue = stdValue = 0
+                else:
+                    minValue = float(np.nanmin(data))
+                    maxValue = float(np.nanmax(data))
+                    meanValue = float(np.nanmean(data))
+                    stdValue = float(np.nanstd(data))
                 ds.GetRasterBand(1).SetStatistics(minValue, maxValue, meanValue, stdValue)
             # ---
 
@@ -256,3 +263,4 @@ def Numpy2Gdal(data, gt, prj, filename, save_nodata_as=-9999):
         return Numpy2AAIGrid(data, gt, prj, filename, save_nodata_as)
     else:
         return ""
+
