@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------
-# Licence:
+# MIT License:
 # Copyright (c) 2012-2021 Luzzi Valerio
 #
 # The above copyright notice and this permission notice shall be
@@ -33,12 +33,15 @@ import site
 from osgeo import gdal
 from osgeo import osr, ogr
 from pyproj import CRS
-from .filesystem import justext, juststem, forceext, justpath, strtofile, filetostr, md5text, listify
-from .module_open import OpenRaster
+from .filesystem import justext, juststem, forceext, justpath
+from .filesystem import strtofile, md5text, listify, normshape, parse_shape_path
+from .module_open import OpenRaster, get
 from .module_open import OpenShape
-from .module_s3 import isfile
+from .module_s3 import isfile, isshape, israster
 from .module_log import Logger
 from Levenshtein import distance
+from .module_types import parseFloat, isarray, isstring
+from .module_http import nominatim_search
 
 shpext = ("shp", "dbf", "shx", "prj", "qpj", "qml", "qix", "idx", "dat", "sbn", "sbx", "fbn", "fbx", "ain", "aih",
           "atx", "qlr", "mta", "qmd", "cpg")
@@ -185,10 +188,10 @@ def AutoIdentify(wkt):
     # get the file pe_hash_list.json from package data
     code = None
 
-    if isfile(wkt) and wkt.endswith(".tif"):
+    if israster(wkt):
         wkt = OpenRaster(wkt).GetProjection()
-    elif isfile(wkt) and isfile(forceext(wkt, "prj")):
-        wkt = filetostr(forceext(wkt, "prj"))
+    elif isshape(wkt) and isfile(forceext(wkt, "prj")):
+        wkt = get(forceext(wkt, "prj"))
     elif isinstance(wkt, int):
         wkt = f"EPSG:{wkt}"
         return wkt
@@ -312,13 +315,13 @@ def GetSpatialRef(filename):
             srs.ImportFromWkt(wkt)
             srs.AutoIdentifyEPSG()
 
-    elif isinstance(filename, str) and isfile(filename) and filename.lower().endswith(".shp"):
+    elif isinstance(filename, str) and isshape(filename):
         ds = OpenShape(filename)
         if ds:
             srs = ds.GetLayer().GetSpatialRef()
             srs.AutoIdentifyEPSG()
 
-    elif isinstance(filename, str) and isfile(filename) and filename.lower().endswith(".tif"):
+    elif isinstance(filename, str) and israster(filename):
         ds = OpenRaster(filename)
         if ds:
             wkt = ds.GetProjection()
@@ -326,7 +329,7 @@ def GetSpatialRef(filename):
             srs.ImportFromWkt(wkt)
             srs.AutoIdentifyEPSG()
     else:
-        srs = osr.SpatialReference()
+        srs = None
 
     return srs
 
@@ -360,27 +363,6 @@ def GetGeometryType(filename):
     return None
 
 
-def Rectangle(minx, miny, maxx, maxy, deltaperc=0.0):
-    """
-    Rectangle - create ogr polygon from bbox
-    """
-    deltaperc = min(1.0, max(0.0, abs(deltaperc)))
-    width = abs(maxx - minx)
-    height = abs(maxy - miny)
-    deltax = width * deltaperc
-    deltay = height * deltaperc
-    ring = ogr.Geometry(ogr.wkbLinearRing)
-    ring.AddPoint_2D(minx-deltax, miny-deltay)
-    ring.AddPoint_2D(maxx+deltax, miny-deltay)
-    ring.AddPoint_2D(maxx+deltax, maxy+deltay)
-    ring.AddPoint_2D(minx-deltax, maxy+deltay)
-    ring.AddPoint_2D(minx-deltax, miny-deltay)
-    # Create polygon
-    poly = ogr.Geometry(ogr.wkbPolygon)
-    poly.AddGeometry(ring)
-    return poly
-
-
 def TransformBBOX(bbox, s_srs=None, t_srs=None):
     """
     TransformBBOX
@@ -402,9 +384,10 @@ def TransformBBOX(bbox, s_srs=None, t_srs=None):
     # t_minx, t_maxx, t_miny, t_maxy = rect.GetEnvelope()
     # transformed_bbox = (t_minx, t_miny, t_maxx, t_maxy)
     s_minx, s_miny, s_maxx, s_maxy = bbox
-    minx, miny, maxx, maxy = transform.TransformBounds(s_minx, s_miny, s_maxx, s_maxy, 2)
+    minx, miny, maxx, maxy = transform.TransformBounds(
+        s_minx, s_miny, s_maxx, s_maxy, 2)
 
-     # patch for EPSG:6876
+    # patch for EPSG:6876
     if SameSpatialRef(t_srs, GetSpatialRef(6876)):
         minx, miny, maxx, maxy = miny, minx, maxy, maxx
     elif SameSpatialRef(t_srs, GetSpatialRef(3035)):
@@ -413,20 +396,134 @@ def TransformBBOX(bbox, s_srs=None, t_srs=None):
     return minx, miny, maxx, maxy
 
 
+
+def Rectangle(minx, miny, maxx, maxy, deltaperc=0.0):
+    """
+    Rectangle - create ogr polygon from bbox
+    """
+    deltaperc = min(1.0, max(0.0, abs(deltaperc)))
+    width = abs(maxx - minx)
+    height = abs(maxy - miny)
+    deltax = width * deltaperc
+    deltay = height * deltaperc
+    ring = ogr.Geometry(ogr.wkbLinearRing)
+    ring.AddPoint_2D(minx-deltax, miny-deltay)
+    ring.AddPoint_2D(maxx+deltax, miny-deltay)
+    ring.AddPoint_2D(maxx+deltax, maxy+deltay)
+    ring.AddPoint_2D(minx-deltax, maxy+deltay)
+    ring.AddPoint_2D(minx-deltax, miny-deltay)
+    # Create polygon
+    poly = ogr.Geometry(ogr.wkbPolygon)
+    poly.AddGeometry(ring)
+    return poly
+
+
+def PolygonFrom(extent, delta=0.0, s_srs=None, t_srs=None):
+    """
+    PolygonFrom
+    """
+    geom = None
+    if not extent:
+        return None
+    elif isinstance(extent, ogr.Geometry) and\
+            extent.GetGeometryName() in ("POLYGON", "MULTIPOLYGON"):
+        geom = extent
+    elif isinstance(extent, ogr.Feature):
+        geom = PolygonFrom(extent.GetGeometryRef(), delta, s_srs, t_srs)
+    # array di 4 stringhe
+    elif isstring(extent) and len(listify(extent))==4 and\
+        all([isinstance(parseFloat(item), float) for item in listify(extent)]):
+            extent = [float(item) for item in listify(extent)]
+            geom = Rectangle(*extent)
+    # array di 4 float
+    elif isarray(extent) and len(extent) == 4:
+        extent = [float(item) for item in extent]
+        geom = Rectangle(*extent)
+    # wkt string
+    elif isstring(extent) and\
+        (extent.startswith("POLYGON") or extent.startswith("MULTIPOLYGON")):
+        try:
+            geom = ogr.CreateGeometryFromWkt(extent)
+        except Exception as e:
+            Logger.error(f"PolygonFrom: {e}")     
+    # from raster extent
+    elif israster(extent):
+        minx, miny, maxx, maxy = GetExtent(extent)
+        geom = Rectangle(minx, miny, maxx, maxy)
+        s_srs = s_srs if s_srs else GetSpatialRef(extent)
+
+    # from shape extent
+    elif isshape(extent):
+        filename, fieldname, fids = parse_shape_path(extent)
+        ds = OpenShape(filename)
+        if ds:
+            layer = ds.GetLayer()
+            if fieldname and fids is not None:
+                features = [layer.GetFeature(fid) for fid in listify(fids)]
+                geoms = [feature.GetGeometryRef() for feature in features if feature]
+                # merge all the geometries
+                if len(geoms) > 0:
+                    geom = geoms[0].Clone()
+                    for g in geoms[1:]:
+                        geom = geom.Union(g)
+            else:
+                minx, maxx, miny, maxy = layer.GetExtent()
+                geom = Rectangle(minx, miny, maxx, maxy)
+    
+        # get the spatial reference
+        s_srs = s_srs if s_srs else GetSpatialRef(extent)
+        ds = None
+    # from nominatim query
+    elif isstring(extent):
+        geojson = nominatim_search(extent)
+        if geojson and "geotext" in geojson and "POLYGON" in geojson["geotext"]:
+            wkt = geojson["geotext"]
+            geom = PolygonFrom(wkt, delta, s_srs, t_srs)
+        elif geojson and "boundingbox" in geojson:
+            extent = geojson["boundingbox"]
+            minlat, maxlat, minlon, maxlon = extent
+            extent = (minlon, minlat, maxlon, maxlat)
+            geom = PolygonFrom(extent, delta, s_srs, t_srs)
+
+    # Increase the extent by a delta percentage
+    if geom and abs(delta) > 0:
+        # delta is a percentage of the width and height
+        minx, maxx, miny, maxy = geom.GetEnvelope()
+        width = abs(maxx - minx)
+        height = abs(maxy - miny)
+        sign = 1 if delta > 0 else -1
+        delta = min(1.0, max(0.0, abs(delta)))
+        buff = min(width, height) * delta * sign
+        geom = geom.Buffer(buff)
+
+    # Transform the geometry
+    if s_srs and t_srs and not SameSpatialRef(s_srs, t_srs):
+        s_srs = GetSpatialRef(s_srs)
+        t_srs = GetSpatialRef(t_srs)
+        if s_srs and s_srs.IsGeographic():
+            s_srs.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
+        if t_srs and t_srs.IsGeographic():
+            t_srs.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
+        transform = osr.CoordinateTransformation(s_srs, t_srs)
+        geom.Transform(transform)
+
+    return geom
+
+
 def GetExtent(filename, t_srs=None):
     """
     GetExtent
     """
     s_srs = None
     minx, miny, maxx, maxy = 0, 0, 0, 0
-    ext = justext(f"{filename}").lower()
-    if isinstance(filename, str) and not isfile(filename):
+    if isinstance(filename, str) and not isfile(normshape(filename)):
         # replace ; with , in case of a list of coordinates
         filename = filename.replace(";", ",")
         # replace single space with , in case of a list of coordinates
         filename = re.sub(r"\s+", ",", filename)
         arr = listify(filename)
-        minx, miny, maxx, maxy = [float(item) for item in arr] if len(arr) == 4 else [0, 0, 0, 0]
+        minx, miny, maxx, maxy = [float(item) for item in arr] if len(
+            arr) == 4 else [0, 0, 0, 0]
         s_srs = GetSpatialRef(4326)
     elif isinstance(filename, (list, tuple)) and len(filename) == 4:
         minx, miny, maxx, maxy = filename
@@ -434,7 +531,7 @@ def GetExtent(filename, t_srs=None):
     elif isinstance(filename, ogr.Geometry):
         minx, maxx, miny, maxy = filename.GetEnvelope()
         s_srs = filename.GetSpatialReference()
-    elif ext == "tif":
+    elif israster(filename):
         ds = OpenRaster(filename)
         if ds:
             "{xmin} {ymin} {xmax} {ymax}"
@@ -449,22 +546,26 @@ def GetExtent(filename, t_srs=None):
             s_srs.ImportFromWkt(wkt)
             ds = None
 
-    elif ext in ("shp", "dbf"):
+    elif isshape(filename):
 
-        filename = forceext(filename, "shp")
-        # driver = ogr.GetDriverByName("ESRI Shapefile")
-        # ds = driver.Open(filename, 0)
         ds = OpenShape(filename, 0)
         if ds:
             layer = ds.GetLayer()
-            minx, maxx, miny, maxy = layer.GetExtent()
             s_srs = layer.GetSpatialRef()
+            filename, fieldname, fid = parse_shape_path(filename)
+            if fieldname and fieldname.lower() == "fid":
+                feature = layer.GetFeature(fid)
+                minx, maxx, miny, maxy = feature.GetGeometryRef().GetEnvelope()
+            else:
+                minx, maxx, miny, maxy = layer.GetExtent()
+
             ds = None
 
     if t_srs and not SameSpatialRef(s_srs, t_srs):
 
-        minx, miny, maxx, maxy = TransformBBOX([minx, miny, maxx, maxy], s_srs, t_srs)
-        #print(f"GetExtent: {minx}, {miny}, {maxx}, {maxy}")
+        minx, miny, maxx, maxy = TransformBBOX(
+            [minx, miny, maxx, maxy], s_srs, t_srs)
+        # print(f"GetExtent: {minx}, {miny}, {maxx}, {maxy}")
 
     return minx, miny, maxx, maxy
 
@@ -511,7 +612,7 @@ def find_PROJ_LIB():
         pathnames += glob.glob(root + "/**/proj.db", recursive=True)
         if len(pathnames):
             break
-    return justpath(pathnames[0]) if len(pathnames)>0 else ""
+    return justpath(pathnames[0]) if len(pathnames) > 0 else ""
 
 
 def find_GDAL_DATA():
@@ -524,7 +625,7 @@ def find_GDAL_DATA():
         pathnames += glob.glob(root + "/**/gt_datum.csv", recursive=True)
         if len(pathnames):
             break
-    return justpath(pathnames[0]) if len(pathnames)>0 else ""
+    return justpath(pathnames[0]) if len(pathnames) > 0 else ""
 
 
 def CreateRectangleShape(minx, miny, maxx, maxy, srs, fileshp="tempxy...."):
