@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------
-# Licence:
+# MIT License:
 # Copyright (c) 2012-2022 Luzzi Valerio
 #
 # The above copyright notice and this permission notice shall be
@@ -59,6 +59,23 @@ def isfile(filename):
     return False
 
 
+def israster(pathname):
+    """
+    israster
+    """
+    return pathname and isfile(pathname) and justext(pathname).lower() in ("tif","tiff")
+
+
+def isshape(filename):
+    """
+    isshape
+    """
+    if isinstance(filename, str):
+        filename = normshape(filename)
+        return isfile(filename) and ".shp" in filename.lower()
+    return False
+
+
 def get_bucket_name_key(uri):
     """
     get_bucket_name_key - get bucket name and key name from uri
@@ -77,6 +94,8 @@ def get_bucket_name_key(uri):
         _, _, bucket_name, key_name = uri.split("/", 3)
     elif uri.startswith("https://s3.amazonaws.com/"):
         _, _, bucket_name, key_name = uri.split("/", 3)
+    elif uri.startswith("https://s3."):
+        _, _, bucket_name, key_name = uri.split("/", 3)
     else:
         bucket_name, key_name = None, uri
     return bucket_name, key_name
@@ -167,7 +186,10 @@ def tempname4S3(uri):
     elif uri.startswith("/vsis3/"):
         tmp = uri.replace("/vsis3/", dest_folder + "/")
     else:
-        tmp = dest_folder + "/" + uri
+        _, path = os.path.splitdrive(uri)
+        tmp = normpath(dest_folder + "/" + path)
+    
+    os.makedirs(justpath(tmp), exist_ok=True)
     return tmp
 
 
@@ -371,10 +393,11 @@ def copy(src, dst=None, client=None):
     """
     copy
     """
-    if isinstance(src, (tuple,list)) and dst is None:
+    if isinstance(src, (tuple, list)): # and dst is None:
         return [copy(file, client=client) for file in src]
 
-    dst = dst if dst else tempfilename(prefix="s3/", suffix=f".{justext(src)}")
+    #dst = dst if dst else tempfilename(prefix="s3/", suffix=f".{justext(src)}")
+    dst = dst if dst else tempname4S3(src)
     # if the source and destination are the same file do nothing
     if src and dst and os.path.isfile(src) and os.path.abspath(src) == os.path.abspath(dst):
         return dst
@@ -405,7 +428,7 @@ def copy(src, dst=None, client=None):
     # Finally
     # if the source is a shapefile or a tiff file, copy the related files
     exts = []
-    if src.endswith(".shp"):
+    if ".shp" in src.lower():
         exts = list(shpext)
         exts.remove("shp")
     elif src.endswith(".tif"):
@@ -443,11 +466,12 @@ def move(src, dst, client=None):
     exts = []
     if src.endswith(".shp"):
         exts = list(shpext)
+        exts.remove("shp")
     elif src.endswith(".tif"):
         exts = ["tfw", "jpw", "prj", "aux.xml"]
         
     for ext in exts:
-        move(forceext(src,ext), forceext(dst,ext), client=client)
+        move(forceext(src, ext), forceext(dst, ext), client=client)
 
     return dst
 
