@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------
-# Licence:
+# MIT License:
 # Copyright (c) 2012-2024 Luzzi Valerio
 #
 # The above copyright notice and this permission notice shall be
@@ -23,8 +23,8 @@
 # Created:     20/06/2024
 # -------------------------------------------------------------------------------
 import xmltodict
-from .filesystem import isshape, forceext, listify
-from .module_s3 import isfile
+from .filesystem import forceext, listify
+from .module_s3 import isfile, isshape
 from .module_ogr import GetSpatialRef, AutoIdentify, GetExtent
 
 def parseXML(filename):
@@ -52,7 +52,7 @@ def parseQMD(filename):
     metadata = parseXML(fileqmd)
     if metadata:
         qgis = metadata["qgis"] if "qgis" in metadata else {}
-        if qgis:
+        if qgis and "keywords" in qgis:
             keywords = listify(qgis["keywords"])
             for item in keywords:
                 key = item["@vocabulary"]
